@@ -3,16 +3,21 @@
 
 import os
 import re
+import sys
+import pathlib
 from random import *
 
-from PyQt5.QtCore import Qt, QRect, QStandardPaths, QObject, QRegExp, QDir
-from PyQt5.QtCore import QUrl, QTimer
+from PyQt5.QtCore import Qt, QRect, QStandardPaths, QObject, QProcess, QRegExp
+from PyQt5.QtCore import QDir, QUrl, QTimer
 from PyQt5.QtGui import QBrush, QIcon, QPainter, QColor, QImage, QPixmap
 from PyQt5.QtGui import QDesktopServices
-from PyQt5.QtWidgets import qApp, QFileDialog, QTextEdit
+from PyQt5.QtWidgets import qApp, QFileDialog
 
 from manuskript.enums import Outline
 
+import logging
+LOGGER = logging.getLogger(__name__)
+
 # Used to detect multiple connections
 AUC = Qt.AutoConnection | Qt.UniqueConnection
 MW = None
@@ -23,6 +28,14 @@ def wordCount(text):
     t = [l for l in t if l]
     return len(t)
 
+def charCount(text, use_spaces = True):
+    t = text.strip()
+
+    if not use_spaces:
+        t = t.replace(" ", "")
+
+    return len(t)
+
 validate_ok = lambda *args, **kwargs: True
 def uiParse(input, default, converter, validator=validate_ok):
     """
@@ -442,5 +455,126 @@ def inspect():
             s.function))
         print("   " + "".join(s.code_context))
 
+
+def search(searchRegex, text):
+    """
+    Search all occurrences of a regex in a text.
+
+    :param searchRegex:    a regex object with the search to perform
+    :param text:            text to search on
+    :return:                list of tuples (startPos, endPos)
+    """
+    if text is not None:
+        return [(m.start(), m.end(), getSearchResultContext(text, m.start(), m.end())) for m in searchRegex.finditer(text)]
+    else:
+        return []
+
+def getSearchResultContext(text, startPos, endPos):
+    matchSize = endPos - startPos
+    maxContextSize = max(matchSize, 600)
+    extraContextSize = int((maxContextSize - matchSize) / 2)
+    separator = "[...]"
+
+    context = ""
+
+    i = startPos - 1
+    while i > 0 and (startPos - i) < extraContextSize and text[i] != '\n':
+        i -= 1
+    contextStartPos = i
+    if i > 0:
+        context += separator + " "
+    context += text[contextStartPos:startPos].replace('\n', '')
+
+    context += '<b>' + text[startPos:endPos].replace('\n', '') + '</b>'
+
+    i = endPos
+    while i < len(text) and (i - endPos) < extraContextSize and text[i] != '\n':
+        i += 1
+    contextEndPos = i
+
+    context += text[endPos:contextEndPos].replace('\n', '')
+    if i < len(text):
+        context += " " + separator
+
+    return context
+
+
+# Based on answer by jfs at:
+#  https://stackoverflow.com/questions/3718657/how-to-properly-determine-current-script-directory
+def getManuskriptPath(follow_symlinks=True):
+    """Used to obtain the path Manuskript is located at."""
+    if getattr(sys, 'frozen', False): # py2exe, PyInstaller, cx_Freeze
+        path = os.path.abspath(sys.executable)
+    else:
+        import inspect
+        path = inspect.getabsfile(getManuskriptPath) + "/../.."
+    if follow_symlinks:
+        path = os.path.realpath(path)
+    return os.path.dirname(path)
+
+# Based on answer by kagronik at:
+#   https://stackoverflow.com/questions/14989858/get-the-current-git-hash-in-a-python-script
+def getGitRevision(base_path):
+    """Get git revision without relying on external processes or libraries."""
+    git_dir = pathlib.Path(base_path) / '.git'
+    if not git_dir.exists():
+        return None
+
+    with (git_dir / 'HEAD').open('r') as head:
+        ref = head.readline().split(' ')[-1].strip()
+
+    with (git_dir / ref).open('r') as git_hash:
+        return git_hash.readline().strip()
+
+def getGitRevisionAsString(base_path, short=False):
+    """Catches errors and presents a nice string."""
+    try:
+        rev = getGitRevision(base_path)
+        if rev is not None:
+            if short:
+                rev = rev[:7]
+            return "#" + rev
+        else:
+            return ""  # not a git repository
+    except Exception as e:
+        LOGGER.warning("Failed to obtain Git revision: %s", e)
+        return "#ERROR"
+
+def showInFolder(path, open_file_as_fallback=False):
+    '''
+    Show a file or folder in explorer/finder, highlighting it where possible.
+    Source: https://stackoverflow.com/a/46019091/3388962
+    '''
+    path = os.path.abspath(path)
+    dirPath = path if os.path.isdir(path) else os.path.dirname(path)
+    if sys.platform == 'win32':
+        args = []
+        args.append('/select,')
+        args.append(QDir.toNativeSeparators(path))
+        if QProcess.startDetached('explorer', args):
+            return True
+    elif sys.platform == 'darwin':
+        args = []
+        args.append('-e')
+        args.append('tell application "Finder"')
+        args.append('-e')
+        args.append('activate')
+        args.append('-e')
+        args.append('select POSIX file "%s"' % path)
+        args.append('-e')
+        args.append('end tell')
+        args.append('-e')
+        args.append('return')
+        if not QProcess.execute('/usr/bin/osascript', args):
+            return True
+        #if not QtCore.QProcess.execute('/usr/bin/open', [dirPath]):
+        #    return
+    # TODO: Linux is not implemented. It has many file managers (nautilus, xdg-open, etc.)
+    # each of which needs special ways to highlight a file in a file manager window.
+
+    # Fallback.
+    return QDesktopServices.openUrl(QUrl(path if open_file_as_fallback else dirPath))
+
+
 # Spellchecker loads writablePath from this file, so we need to load it after they get defined
 from manuskript.functions.spellchecker import Spellchecker
