@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # --!-- coding: utf8 --!--
-import imp
+import importlib
 import os
 import re
 
@@ -9,13 +9,14 @@ from PyQt5.QtCore import (pyqtSignal, QSignalMapper, QTimer, QSettings, Qt, QPoi
                           QRegExp, QUrl, QSize, QModelIndex)
 from PyQt5.QtGui import QStandardItemModel, QIcon, QColor
 from PyQt5.QtWidgets import QMainWindow, QHeaderView, qApp, QMenu, QActionGroup, QAction, QStyle, QListWidgetItem, \
-    QLabel, QDockWidget, QWidget, QMessageBox
+    QLabel, QDockWidget, QWidget, QMessageBox, QLineEdit
 
 from manuskript import settings
 from manuskript.enums import Character, PlotStep, Plot, World, Outline
-from manuskript.functions import wordCount, appPath, findWidgetsOfClass
+from manuskript.functions import wordCount, appPath, findWidgetsOfClass, openURL, showInFolder
 import manuskript.functions as F
 from manuskript import loadSave
+from manuskript.logging import getLogFilePath
 from manuskript.models.characterModel import characterModel
 from manuskript.models import outlineModel
 from manuskript.models.plotModel import plotModel
@@ -38,6 +39,9 @@ from manuskript.ui.statusLabel import statusLabel
 from manuskript.ui.views.textEditView import textEditView
 from manuskript.functions import Spellchecker
 
+import logging
+LOGGER = logging.getLogger(__name__)
+
 class MainWindow(QMainWindow, Ui_MainWindow):
     # dictChanged = pyqtSignal(str)
 
@@ -129,6 +133,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         self.actCopy.triggered.connect(self.documentsCopy)
         self.actCut.triggered.connect(self.documentsCut)
         self.actPaste.triggered.connect(self.documentsPaste)
+        self.actSearch.triggered.connect(self.doSearch)
         self.actRename.triggered.connect(self.documentsRename)
         self.actDuplicate.triggered.connect(self.documentsDuplicate)
         self.actDelete.triggered.connect(self.documentsDelete)
@@ -175,6 +180,8 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
         # Main Menu:: Tool
         self.actToolFrequency.triggered.connect(self.frequencyAnalyzer)
+        self.actSupport.triggered.connect(self.support)
+        self.actLocateLog.triggered.connect(self.locateLogFile)
         self.actAbout.triggered.connect(self.about)
 
         self.makeUIConnections()
@@ -270,7 +277,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             self.mainEditor
             ]
 
-        while new is not None:
+        while new != None:
             if new in targets:
                 self._lastFocus = new
                 break
@@ -346,6 +353,9 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # Slider importance
         self.updateCharacterImportance(c.ID())
 
+        # POV state
+        self.updateCharacterPOVState(c.ID())
+
         # Character Infos
         self.tblPersoInfos.setRootIndex(index)
 
@@ -366,6 +376,22 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         c = self.mdlCharacter.getCharacterByID(ID)
         self.sldPersoImportance.setValue(int(c.importance()))
 
+    def updateCharacterPOVState(self, ID):
+        c = self.mdlCharacter.getCharacterByID(ID)
+        self.disconnectAll(self.chkPersoPOV.stateChanged, self.lstCharacters.changeCharacterPOVState)
+
+        if c.pov():
+            self.chkPersoPOV.setCheckState(Qt.Checked)
+        else:
+            self.chkPersoPOV.setCheckState(Qt.Unchecked)
+
+        try:
+            self.chkPersoPOV.stateChanged.connect(self.lstCharacters.changeCharacterPOVState, F.AUC)
+            self.chkPersoPOV.setEnabled(len(self.mdlOutline.findItemsByPOV(ID)) == 0)
+        except TypeError:
+            #don't know what's up with this
+            pass
+
     ###############################################################################
     # PLOTS
     ###############################################################################
@@ -480,6 +506,13 @@ class MainWindow(QMainWindow, Ui_MainWindow):
     def documentsPaste(self):
         "Paste clipboard item(s) into selected item."
         if self._lastFocus: self._lastFocus.paste()
+    def doSearch(self):
+        "Do a global search."
+        self.dckSearch.show()
+        self.dckSearch.activateWindow()
+        searchTextInput = self.dckSearch.findChild(QLineEdit, 'searchTextInput')
+        searchTextInput.setFocus()
+        searchTextInput.selectAll()
     def documentsRename(self):
         "Rename selected item."
         if self._lastFocus: self._lastFocus.rename()
@@ -557,14 +590,14 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         If ``loadFromFile`` is False, then it does not load datas from file.
         It assumes that the datas have been populated in a different way."""
         if loadFromFile and not os.path.exists(project):
-            print(self.tr("The file {} does not exist. Has it been moved or deleted?").format(project))
+            LOGGER.warning("The file {} does not exist. Has it been moved or deleted?".format(project))
             F.statusMessage(
                     self.tr("The file {} does not exist. Has it been moved or deleted?").format(project), importance=3)
             return
 
         if loadFromFile:
             # Load empty settings
-            imp.reload(settings)
+            importlib.reload(settings)
             settings.initDefaultValues()
 
             # Load data
@@ -612,7 +645,6 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         self.mdlCharacter.dataChanged.connect(self.startTimerNoChanges)
         self.mdlPlots.dataChanged.connect(self.startTimerNoChanges)
         self.mdlWorld.dataChanged.connect(self.startTimerNoChanges)
-        # self.mdlPersosInfos.dataChanged.connect(self.startTimerNoChanges)
         self.mdlStatus.dataChanged.connect(self.startTimerNoChanges)
         self.mdlLabels.dataChanged.connect(self.startTimerNoChanges)
 
@@ -638,9 +670,6 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # Add project name to Window's name
         self.setWindowTitle(self.projectName() + " - " + self.tr("Manuskript"))
 
-        # Stuff
-        # self.checkPersosID()  # Shouldn't be necessary any longer
-
         # Show main Window
         self.switchToProject()
 
@@ -768,6 +797,10 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             # Remembering the current items (stores outlineItem's ID)
             settings.openIndexes = self.mainEditor.tabSplitter.openIndexes()
 
+            # Call close on the main window to clean children widgets
+            if self.mainEditor:
+                self.mainEditor.close()
+
             # Save data from models
             if settings.saveOnQuit:
                 self.saveDatas()
@@ -823,11 +856,11 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # risk a scenario where the timer somehow triggers a new save while saving.
         self.saveTimerNoChanges.stop()
 
-        if self.currentProject is None:
+        if self.currentProject == None:
             # No UI feedback here as this code path indicates a race condition that happens
             # after the user has already closed the project through some way. But in that
             # scenario, this code should not be reachable to begin with.
-            print("Bug: there is no current project to save.")
+            LOGGER.error("There is no current project to save.")
             return
 
         r = loadSave.saveProject()  # version=0
@@ -838,18 +871,15 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
             feedback = self.tr("Project {} saved.").format(projectName)
             F.statusMessage(feedback, importance=0)
+            LOGGER.info("Project {} saved.".format(projectName))
         else:
             feedback = self.tr("WARNING: Project {} not saved.").format(projectName)
             F.statusMessage(feedback, importance=3)
-
-        # Giving some feedback in console
-        print(feedback)
+            LOGGER.warning("Project {} not saved.".format(projectName))
 
     def loadEmptyDatas(self):
         self.mdlFlatData = QStandardItemModel(self)
         self.mdlCharacter = characterModel(self)
-        # self.mdlPersosProxy = persosProxyModel(self)
-        # self.mdlPersosInfos = QStandardItemModel(self)
         self.mdlLabels = QStandardItemModel(self)
         self.mdlStatus = QStandardItemModel(self)
         self.mdlPlots = plotModel(self)
@@ -862,13 +892,13 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
         # Giving some feedback
         if not errors:
-            print(self.tr("Project {} loaded.").format(project))
+            LOGGER.info("Project {} loaded.".format(project))
             F.statusMessage(
                     self.tr("Project {} loaded.").format(project), 2000)
         else:
-            print(self.tr("Project {} loaded with some errors:").format(project))
+            LOGGER.error("Project {} loaded with some errors:".format(project))
             for e in errors:
-                print(self.tr(" * {} wasn't found in project file.").format(e))
+                LOGGER.error(" * {} wasn't found in project file.".format(e))
             F.statusMessage(
                     self.tr("Project {} loaded with some errors.").format(project), 5000, importance = 3)
 
@@ -933,11 +963,13 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # Characters
         self.lstCharacters.setCharactersModel(self.mdlCharacter)
         self.tblPersoInfos.setModel(self.mdlCharacter)
-
-        self.btnAddPerso.clicked.connect(self.mdlCharacter.addCharacter, F.AUC)
         try:
+            self.btnAddPerso.clicked.connect(self.lstCharacters.addCharacter, F.AUC)
             self.btnRmPerso.clicked.connect(self.lstCharacters.removeCharacter, F.AUC)
+
             self.btnPersoColor.clicked.connect(self.lstCharacters.choseCharacterColor, F.AUC)
+            self.chkPersoPOV.stateChanged.connect(self.lstCharacters.changeCharacterPOVState, F.AUC)
+
             self.btnPersoAddInfo.clicked.connect(self.lstCharacters.addCharacterInfo, F.AUC)
             self.btnPersoRmInfo.clicked.connect(self.lstCharacters.removeCharacterInfo, F.AUC)
         except TypeError:
@@ -1078,7 +1110,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # disconnect only removes one connection at a time.
         while True:
             try:
-                if oldHandler is not None:
+                if oldHandler != None:
                     signal.disconnect(oldHandler)
                 else:
                     signal.disconnect()
@@ -1089,9 +1121,12 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # Break connections for UI elements that were connected in makeConnections()
 
         # Characters
-        self.disconnectAll(self.btnAddPerso.clicked, self.mdlCharacter.addCharacter)
+        self.disconnectAll(self.btnAddPerso.clicked, self.lstCharacters.addCharacter)
         self.disconnectAll(self.btnRmPerso.clicked, self.lstCharacters.removeCharacter)
+
         self.disconnectAll(self.btnPersoColor.clicked, self.lstCharacters.choseCharacterColor)
+        self.disconnectAll(self.chkPersoPOV.stateChanged, self.lstCharacters.changeCharacterPOVState)
+
         self.disconnectAll(self.btnPersoAddInfo.clicked, self.lstCharacters.addCharacterInfo)
         self.disconnectAll(self.btnPersoRmInfo.clicked, self.lstCharacters.removeCharacterInfo)
 
@@ -1147,6 +1182,50 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         r2 = self.geometry()
         win.move(r2.center() - QPoint(r.width()/2, r.height()/2))
 
+    def support(self):
+        openURL("https://github.com/olivierkes/manuskript/wiki/Technical-Support")
+
+    def locateLogFile(self):
+        logfile = getLogFilePath()
+
+        # Make sure we are even logging to a file.
+        if not logfile:
+            QMessageBox(QMessageBox.Information,
+                self.tr("Sorry!"),
+                "<p><b>" +
+                    self.tr("This session is not being logged.") +
+                "</b></p>",
+                QMessageBox.Ok).exec()
+            return
+
+        # Remind user that log files are at their best once they are complete.
+        msg = QMessageBox(QMessageBox.Information,
+            self.tr("A log file is a Work in Progress!"),
+            "<p><b>" +
+                self.tr("The log file \"{}\" will continue to be written to until Manuskript is closed.").format(os.path.basename(logfile)) +
+            "</b></p>" +
+            "<p>" +
+                self.tr("It will now be displayed in your file manager, but is of limited use until you close Manuskript.") +
+            "</p>",
+            QMessageBox.Ok)
+
+        ret = msg.exec()
+
+        # Open the filemanager.
+        if ret == QMessageBox.Ok:
+            if not showInFolder(logfile):
+                # If everything convenient fails, at least make sure the user can browse to its location manually.
+                QMessageBox(QMessageBox.Critical,
+                    self.tr("Error!"),
+                    "<p><b>" +
+                        self.tr("An error was encountered while trying to show the log file below in your file manager.") +
+                    "</b></p>" +
+                    "<p>" +
+                        logfile +
+                    "</p>",
+                    QMessageBox.Ok).exec()
+
+
     def about(self):
         self.dialog = aboutDialog(mw=self)
         self.dialog.setFixedSize(self.dialog.size())
@@ -1355,7 +1434,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         dictionaries = Spellchecker.availableDictionaries()
 
         # Set first run dictionary
-        if settings.dict is None:
+        if settings.dict == None:
             settings.dict = Spellchecker.getDefaultDictionary()
 
         # Check if project dict is unavailable on this machine
@@ -1499,7 +1578,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         self.menuView.addMenu(self.menuMode)
         self.menuView.addSeparator()
 
-        # print("Generating menus with", settings.viewSettings)
+        # LOGGER.debug("Generating menus with %s.", settings.viewSettings)
 
         for mnu, mnud, icon in menus:
             m = QMenu(mnu, self.menuView)
@@ -1566,7 +1645,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             w.cmbPOV.setVisible(val)
 
         # POV in outline view
-        if val is None and Outline.POV in settings.outlineViewColumns:
+        if val == None and Outline.POV in settings.outlineViewColumns:
             settings.outlineViewColumns.remove(Outline.POV)
 
         from manuskript.ui.views.outlineView import outlineView
@@ -1593,7 +1672,8 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             warning2 = self.tr("PyQt {} and Qt {} are in use.").format(qVersion(), PYQT_VERSION_STR)
 
             # Don't translate for debug log.
-            print("WARNING:", warning1, warning2)
+            LOGGER.warning(warning1)
+            LOGGER.warning(warning2)
 
             msg = QMessageBox(QMessageBox.Warning,
                 self.tr("Proceed with import at your own risk"),
