@@ -9,18 +9,24 @@ from PyQt5.QtCore import Qt
 from PyQt5.QtGui import QIcon, QFont
 from PyQt5.QtWidgets import QTextEdit, qApp
 from lxml import etree as ET
+import re
 
 from manuskript import enums
 
+import logging
+LOGGER = logging.getLogger(__name__)
 
 class abstractItem():
 
-    # Enum kept on the class for easier acces
+    # Enum kept on the class for easier access
     enum = enums.Abstract
 
     # Used for XML export
     name = "abstractItem"
 
+    # Regexp from https://stackoverflow.com/questions/8733233/filtering-out-certain-bytes-in-python
+    valid_xml_re = re.compile(u'[^\u0020-\uD7FF\u0009\u000A\u000D\uE000-\uFFFD\U00010000-\U0010FFFF]+')
+
     def __init__(self, model=None, title="", _type="abstract", xml=None, parent=None, ID=None):
 
         self._data = {}
@@ -35,14 +41,17 @@ class abstractItem():
         self._data[self.enum.title] = title
         self._data[self.enum.type] = _type
 
-        if xml is not None:
+        if xml != None:
             self.setFromXML(xml)
 
+        if parent:
+            # add this as a child to the parent, and link to the outlineModel of the parent
+            parent.appendChild(self)
+
         if ID:
             self._data[self.enum.ID] = ID
 
-        if parent:
-            parent.appendChild(self)
+
 
     #######################################################################
     # Model
@@ -50,6 +59,11 @@ class abstractItem():
 
     def setModel(self, model):
         self._model = model
+        if not self.ID():
+            self.getUniqueID()
+        elif model:
+            # if we are setting a model update it's ID
+            self._model.updateAvailableIDs(self.ID())
         for c in self.children():
             c.setModel(model)
 
@@ -101,7 +115,7 @@ class abstractItem():
         return self._data[self.enum.type]
 
     #######################################################################
-    # Parent / Children managment
+    # Parent / Children management
     #######################################################################
 
     def child(self, row):
@@ -131,8 +145,6 @@ class abstractItem():
         self.childItems.insert(row, child)
         child._parent = self
         child.setModel(self._model)
-        if not child.ID():
-            child.getUniqueID()
 
     def removeChild(self, row):
         """
@@ -141,6 +153,9 @@ class abstractItem():
         @return: the removed abstractItem
         """
         r = self.childItems.pop(row)
+        # Disassociate the child from its parent and the model.
+        r._parent = None
+        r.setModel(None)
         return r
 
     def parent(self):
@@ -188,7 +203,7 @@ class abstractItem():
     ###############################################################################
 
     def getUniqueID(self, recursive=False):
-        self.setData(self.enum.ID, self._model.rootItem.findUniqueID())
+        self.setData(self.enum.ID, self._model.requestNewID())
 
         if recursive:
             for c in self.children():
@@ -202,7 +217,7 @@ class abstractItem():
         self.IDs = self.listAllIDs()
 
         if max([self.IDs.count(i) for i in self.IDs if i]) != 1:
-            print("WARNING ! There are some items with same IDs:", [i for i in self.IDs if i and self.IDs.count(i) != 1])
+            LOGGER.warning("There are some items with overlapping IDs: %s", [i for i in self.IDs if i and self.IDs.count(i) != 1])
 
         def checkChildren(item):
             for c in item.children():
@@ -219,14 +234,6 @@ class abstractItem():
             IDs.extend(c.listAllIDs())
         return IDs
 
-    def findUniqueID(self):
-        IDs = [int(i) for i in self.IDs]
-        k = 1
-        while k in IDs:
-            k += 1
-        self.IDs.append(str(k))
-        return str(k)
-
     #######################################################################
     # Data
     #######################################################################
@@ -243,6 +250,10 @@ class abstractItem():
         # Setting data
         self._data[column] = data
 
+        # The _model will be none during splitting
+        if self._model and column == self.enum.ID:
+            self._model.updateAvailableIDs(data)
+
         # Emit signal
         self.emitDataChanged(cols=[column]) # new in 0.5.0
 
@@ -255,6 +266,9 @@ class abstractItem():
     # We want to force some data even if they're empty
     XMLForce = []
 
+    def cleanTextForXML(self, text):
+        return self.valid_xml_re.sub('', text)
+
     def toXML(self):
         """
         Returns a string containing the item (and children) in XML.
@@ -269,7 +283,7 @@ class abstractItem():
                 continue
             val = self.data(attrib)
             if val or attrib in self.XMLForce:
-                item.set(attrib.name, str(val))
+                item.set(attrib.name, self.cleanTextForXML(str(val)))
 
         # Saving lastPath
         item.set("lastPath", self._lastPath)
