@@ -8,10 +8,13 @@ from PyQt5.QtGui import QFont, QIcon
 from PyQt5.QtWidgets import qApp
 from lxml import etree as ET
 from manuskript.models.abstractItem import abstractItem
+from manuskript.models.searchableItem import searchableItem
 from manuskript import enums
 from manuskript import functions as F
 from manuskript import settings
 from manuskript.converters import HTML2PlainText
+from manuskript.searchLabels import OutlineSearchLabels
+from manuskript.enums import Outline, Model
 
 try:
     locale.setlocale(locale.LC_ALL, '')
@@ -20,8 +23,10 @@ except:
     # number formatting
     pass
 
+import logging
+LOGGER = logging.getLogger(__name__)
 
-class outlineItem(abstractItem):
+class outlineItem(abstractItem, searchableItem):
 
     enum = enums.Outline
 
@@ -30,6 +35,7 @@ class outlineItem(abstractItem):
 
     def __init__(self, model=None, title="", _type="folder", xml=None, parent=None, ID=None):
         abstractItem.__init__(self, model, title, _type, xml, parent, ID)
+        searchableItem.__init__(self, OutlineSearchLabels)
 
         self.defaultTextType = None
         if not self._data.get(self.enum.compile):
@@ -80,6 +86,9 @@ class outlineItem(abstractItem):
     def wordCount(self):
         return self._data.get(self.enum.wordCount, 0)
 
+    def charCount(self):
+        return self._data.get(self.enum.charCount, 0)
+
     def __str__(self):
         return "{id}: {folder}{title}{children}".format(
             id=self.ID(),
@@ -89,6 +98,9 @@ class outlineItem(abstractItem):
             )
 
     __repr__ = __str__
+    
+    def charCount(self):
+        return self._data.get(self.enum.charCount, 0)
 
     #######################################################################
     # Data
@@ -119,7 +131,7 @@ class outlineItem(abstractItem):
 
         elif role == Qt.FontRole:
             f = QFont()
-            if column == E.wordCount and self.isFolder():
+            if (column == E.wordCount or column == E.charCount) and self.isFolder():
                 f.setItalic(True)
             elif column == E.goal and self.isFolder() and not self.data(E.setGoal):
                 f.setItalic(True)
@@ -140,7 +152,7 @@ class outlineItem(abstractItem):
 
         # Checking if we will have to recount words
         updateWordCount = False
-        if column in [E.wordCount, E.goal, E.setGoal]:
+        if column in [E.wordCount, E.charCount, E.goal, E.setGoal]:
             updateWordCount = not column in self._data or self._data[column] != data
 
         # Stuff to do before
@@ -153,7 +165,9 @@ class outlineItem(abstractItem):
         # Stuff to do afterwards
         if column == E.text:
             wc = F.wordCount(data)
+            cc = F.charCount(data, settings.countSpaces)
             self.setData(E.wordCount, wc)
+            self.setData(E.charCount, cc)
 
         if column == E.compile:
             # Title changes when compile changes
@@ -178,13 +192,11 @@ class outlineItem(abstractItem):
 
     def removeChild(self, row):
         r = abstractItem.removeChild(self, row)
-        # Might be causing segfault when updateWordCount emits dataChanged
-        self.updateWordCount(emit=False)
+        self.updateWordCount()
         return r
 
-    def updateWordCount(self, emit=True):
-        """Update word count for item and parents.
-        If emit is False, no signal is emitted (sometimes cause segfault)"""
+    def updateWordCount(self):
+        """Update word count for item and parents."""
         if not self.isFolder():
             setGoal = F.toInt(self.data(self.enum.setGoal))
             goal = F.toInt(self.data(self.enum.goal))
@@ -197,9 +209,12 @@ class outlineItem(abstractItem):
 
         else:
             wc = 0
+            cc = 0
             for c in self.children():
                 wc += F.toInt(c.data(self.enum.wordCount))
+                cc += F.toInt(c.data(self.enum.charCount))
             self._data[self.enum.wordCount] = wc
+            self._data[self.enum.charCount] = cc
 
             setGoal = F.toInt(self.data(self.enum.setGoal))
             goal = F.toInt(self.data(self.enum.goal))
@@ -219,12 +234,12 @@ class outlineItem(abstractItem):
             else:
                 self.setData(self.enum.goalPercentage, "")
 
-        if emit:
-            self.emitDataChanged([self.enum.goal, self.enum.setGoal,
-                                  self.enum.wordCount, self.enum.goalPercentage])
+        self.emitDataChanged([self.enum.goal, self.enum.setGoal,
+                              self.enum.wordCount, self.enum.charCount,
+                              self.enum.goalPercentage])
 
         if self.parent():
-            self.parent().updateWordCount(emit)
+            self.parent().updateWordCount()
 
     def stats(self):
         wc = self.data(enums.Outline.wordCount)
@@ -346,8 +361,7 @@ class outlineItem(abstractItem):
 
         return lst
 
-    def findItemsContaining(self, text, columns, mainWindow=F.mainWindow(),
-                            caseSensitive=False, recursive=True):
+    def findItemsContaining(self, text, columns, mainWindow=F.mainWindow(), caseSensitive=False, recursive=True):
         """Returns a list if IDs of all subitems
         containing ``text`` in columns ``columns``
         (being a list of int).
@@ -360,19 +374,17 @@ class outlineItem(abstractItem):
 
         return lst
 
-    def itemContains(self, text, columns, mainWindow=F.mainWindow(),
-                     caseSensitive=False):
+    def itemContains(self, text, columns, mainWindow=F.mainWindow(), caseSensitive=False):
         lst = []
         text = text.lower() if not caseSensitive else text
         for c in columns:
-
             if c == self.enum.POV and self.POV():
-                c = mainWindow.mdlCharacter.getCharacterByID(self.POV())
-                if c:
-                    searchIn = c.name()
+                character = mainWindow.mdlCharacter.getCharacterByID(self.POV())
+                if character:
+                    searchIn = character.name()
                 else:
                     searchIn = ""
-                    print("Character POV not found:", self.POV())
+                    LOGGER.error("Character POV not found: %s", self.POV())
 
             elif c == self.enum.status:
                 searchIn = mainWindow.mdlStatus.item(F.toInt(self.status()), 0).text()
@@ -384,7 +396,6 @@ class outlineItem(abstractItem):
                 searchIn = self.data(c)
 
             searchIn = searchIn.lower() if not caseSensitive else searchIn
-
             if text in searchIn:
                 if not self.ID() in lst:
                     lst.append(self.ID())
@@ -470,6 +481,7 @@ class outlineItem(abstractItem):
 
     # We don't want to write some datas (computed)
     XMLExclude = [enums.Outline.wordCount,
+                  enums.Outline.charCount,
                   enums.Outline.goal,
                   enums.Outline.goalPercentage,
                   enums.Outline.revisions]
@@ -483,7 +495,7 @@ class outlineItem(abstractItem):
         for r in rev:
             revItem = ET.Element("revision")
             revItem.set("timestamp", str(r[0]))
-            revItem.set("text", r[1])
+            revItem.set("text", self.cleanTextForXML(r[1]))
             item.append(revItem)
 
         return item
@@ -505,3 +517,39 @@ class outlineItem(abstractItem):
         for child in root:
             if child.tag == "revision":
                 self.appendRevision(child.attrib["timestamp"], child.attrib["text"])
+
+    #######################################################################
+    # Search
+    #######################################################################
+    def searchModel(self):
+        return Model.Outline
+
+    def searchID(self):
+        return self.data(Outline.ID)
+
+    def searchTitle(self, column):
+        return self.title()
+
+    def searchPath(self, column):
+        return [self.translate("Outline")] + self.path().split(' > ') + [self.translate(self.searchColumnLabel(column))]
+
+    def searchData(self, column):
+        mainWindow = F.mainWindow()
+
+        searchData = None
+
+        if column == self.enum.POV and self.POV():
+            character = mainWindow.mdlCharacter.getCharacterByID(self.POV())
+            if character:
+                searchData = character.name()
+
+        elif column == self.enum.status:
+            searchData = mainWindow.mdlStatus.item(F.toInt(self.status()), 0).text()
+
+        elif column == self.enum.label:
+            searchData = mainWindow.mdlLabels.item(F.toInt(self.label()), 0).text()
+
+        else:
+            searchData = self.data(column)
+
+        return searchData
