@@ -59,31 +59,38 @@ class settingsWindow(QWidget, Ui_Settings):
         self.lstMenu.setMaximumWidth(140)
         self.lstMenu.setMinimumWidth(140)
 
+        lowerKeys = [i.lower() for i in list(QStyleFactory.keys())]
+
         # General
         self.cmbStyle.addItems(list(QStyleFactory.keys()))
-        self.cmbStyle.setCurrentIndex(
-            [i.lower() for i in list(QStyleFactory.keys())]
-            .index(qApp.style().objectName()))
+
+        try:
+            self.cmbStyle.setCurrentIndex(lowerKeys.index(qApp.style().objectName()))
+        except ValueError:
+            self.cmbStyle.setCurrentIndex(0)
+
         self.cmbStyle.currentIndexChanged[str].connect(self.setStyle)
 
         self.cmbTranslation.clear()
         tr = OrderedDict()
         tr["English"] = ""
         tr["Arabic (Saudi Arabia)"] = "manuskript_ar_SA.qm"
-        tr["Deutsch"] = "manuskript_de.qm"
+        tr["German"] = "manuskript_de.qm"
         tr["English (Great Britain)"] = "manuskript_en_GB.qm"
-        tr["Español"] = "manuskript_es.qm"
+        tr["Spanish"] = "manuskript_es.qm"
         tr["Persian"] = "manuskript_fa.qm"
-        tr["Français"] = "manuskript_fr.qm"
+        tr["French"] = "manuskript_fr.qm"
         tr["Hungarian"] = "manuskript_hu.qm"
         tr["Indonesian"] = "manuskript_id.qm"
         tr["Italian"] = "manuskript_it.qm"
         tr["Japanese"] = "manuskript_ja.qm"
+        tr["Korean"] = "manuskript_ko.qm"
         tr["Norwegian Bokmål"] = "manuskript_nb_NO.qm"
         tr["Dutch"] = "manuskript_nl.qm"
         tr["Polish"] = "manuskript_pl.qm"
         tr["Portuguese (Brazil)"] = "manuskript_pt_BR.qm"
         tr["Portuguese (Portugal)"] = "manuskript_pt_PT.qm"
+        tr["Romanian"] = "manuskript_ro.qm"
         tr["Russian"] = "manuskript_ru.qm"
         tr["Svenska"] = "manuskript_sv.qm"
         tr["Turkish"] = "manuskript_tr.qm"
@@ -109,6 +116,9 @@ class settingsWindow(QWidget, Ui_Settings):
         self.spnGeneralFontSize.setValue(f.pointSize())
         self.spnGeneralFontSize.valueChanged.connect(self.setAppFontSize)
 
+        self.chkProgressChars.setChecked(settings.progressChars);
+        self.chkProgressChars.stateChanged.connect(self.charSettingsChanged)
+
         self.txtAutoSave.setValidator(QIntValidator(0, 999, self))
         self.txtAutoSaveNoChanges.setValidator(QIntValidator(0, 999, self))
         self.chkAutoSave.setChecked(settings.autoSave)
@@ -162,10 +172,12 @@ class settingsWindow(QWidget, Ui_Settings):
         for item, what, value in [
             (self.rdoTreeItemCount, "InfoFolder", "Count"),
             (self.rdoTreeWC, "InfoFolder", "WC"),
+            (self.rdoTreeCC, "InfoFolder", "CC"),
             (self.rdoTreeProgress, "InfoFolder", "Progress"),
             (self.rdoTreeSummary, "InfoFolder", "Summary"),
             (self.rdoTreeNothing, "InfoFolder", "Nothing"),
             (self.rdoTreeTextWC, "InfoText", "WC"),
+            (self.rdoTreeTextCC, "InfoText", "CC"),
             (self.rdoTreeTextProgress, "InfoText", "Progress"),
             (self.rdoTreeTextSummary, "InfoText", "Summary"),
             (self.rdoTreeTextNothing, "InfoText", "Nothing"),
@@ -178,6 +190,9 @@ class settingsWindow(QWidget, Ui_Settings):
             lambda v: self.lblTreeIconSize.setText("{}x{}".format(v, v)))
         self.sldTreeIconSize.setValue(settings.viewSettings["Tree"]["iconSize"])
 
+        self.chkCountSpaces.setChecked(settings.countSpaces);
+        self.chkCountSpaces.stateChanged.connect(self.countSpacesChanged)
+
         self.rdoCorkOldStyle.setChecked(settings.corkStyle == "old")
         self.rdoCorkNewStyle.setChecked(settings.corkStyle == "new")
         self.rdoCorkNewStyle.toggled.connect(self.setCorkStyle)
@@ -336,6 +351,11 @@ class settingsWindow(QWidget, Ui_Settings):
         sttgs = QSettings(qApp.organizationName(), qApp.applicationName())
         sttgs.setValue("appFontSize", val)
 
+    def charSettingsChanged(self):
+        settings.progressChars = True if self.chkProgressChars.checkState() else False
+
+        self.mw.mainEditor.updateStats()
+
     def saveSettingsChanged(self):
         if self.txtAutoSave.text() in ["", "0"]:
             self.txtAutoSave.setText("1")
@@ -425,10 +445,12 @@ class settingsWindow(QWidget, Ui_Settings):
         for item, what, value in [
             (self.rdoTreeItemCount, "InfoFolder", "Count"),
             (self.rdoTreeWC, "InfoFolder", "WC"),
+            (self.rdoTreeCC, "InfoFolder", "CC"),
             (self.rdoTreeProgress, "InfoFolder", "Progress"),
             (self.rdoTreeSummary, "InfoFolder", "Summary"),
             (self.rdoTreeNothing, "InfoFolder", "Nothing"),
             (self.rdoTreeTextWC, "InfoText", "WC"),
+            (self.rdoTreeTextCC, "InfoText", "CC"),
             (self.rdoTreeTextProgress, "InfoText", "Progress"),
             (self.rdoTreeTextSummary, "InfoText", "Summary"),
             (self.rdoTreeTextNothing, "InfoText", "Nothing"),
@@ -443,6 +465,11 @@ class settingsWindow(QWidget, Ui_Settings):
 
         self.mw.treeRedacOutline.viewport().update()
 
+    def countSpacesChanged(self):
+        settings.countSpaces = True if self.chkCountSpaces.checkState() else False
+
+        self.mw.mainEditor.updateStats()
+
     def setCorkColor(self):
         color = QColor(settings.corkBackground["color"])
         self.colorDialog = QColorDialog(color, self)
