@@ -43,11 +43,19 @@ class corkDelegate(QStyledItemDelegate):
         return QStyledItemDelegate.editorEvent(self, event, model, option, index)
 
     def createEditor(self, parent, option, index):
+        # When the user performs a global search and selects an Outline result (title or summary), the
+        # associated chapter is selected in cork view, triggering a call to this method with the results
+        # list widget set in self.sender(). In this case we store the searched column so we know which
+        # editor should be created.
+        searchedColumn = None
+        if self.sender() is not None and self.sender().objectName() == 'result' and self.sender().currentItem():
+            searchedColumn = self.sender().currentItem().data(Qt.UserRole).column()
+
         self.updateRects(option, index)
 
         bgColor = self.bgColors.get(index, "white")
 
-        if self.mainLineRect.contains(self.lastPos):
+        if searchedColumn == Outline.summarySentence or (self.lastPos is not None and self.mainLineRect.contains(self.lastPos)):
             # One line summary
             self.editing = Outline.summarySentence
             edt = QLineEdit(parent)
@@ -64,7 +72,7 @@ class corkDelegate(QStyledItemDelegate):
             edt.setStyleSheet("background: {}; color: black;".format(bgColor))
             return edt
 
-        elif self.titleRect.contains(self.lastPos):
+        elif searchedColumn == Outline.title or (self.lastPos is not None and self.titleRect.contains(self.lastPos)):
             # Title
             self.editing = Outline.title
             edt = QLineEdit(parent)
